/* AMX Mod X script.
*
*   Shows players IP and IP Subnet description in CIDR Notation
*   Copyright (c) 2007 ahtoh /Russia/
*
*
*   INSTALLATION TIPS:
*
*   1. In order to use this plugin you need NETDB module
*      get it here http://forums.alliedmods.net/showthread.php?t=52326
*      Put netdb_amxx_i386.so to amxmodx\modules dir
*   2. Add line "netdb" to amxmodx\configs\modules.ini file.
*   3. Put show_ip.amxx to amxmodx\plugins dir.
*   4. Add line "show_ip.amxx" to amxmodx\configs\plugins.ini file.
*   5. Copy example ipdesc.ini file to amxmodx\configs dir.
*   6. Edit ipdesc.ini file.
*      Fill it with IP Subnets and their Descriptions.
*      IP Subnet must be in CIDR Notation (like 192.168.0.0/24).
*      Online Subnet Calcilator: http://www.subnet-calculator.com/cidr.php
*      IP Subnet Description will be shown to all players or admins only (if ADMIN_LEVEL defined)
*      when player from corresponding subnet connects or disconnects from the server.
*
*
*   CONSOLE COMMANDS:
*
*   - amx_showip
*       - shows IP, hostname and IP Description for all players on server
*
*
*   CVARS:
*
*   - amx_showip_unknown ("UNKNOWN SUBNET")
*       - IP Subnet Description for unknown IP (not described in ipdesc.ini)
*   - amx_showip_showhost(1)
*       - show hostnames along with IPs (do reverse DNS request)
*
*
*   VERSION HISTORY:
*
*   v1.55 - 04/20/07
*       - Performance optimizations
*
*   v1.5 - 04/05/07
*       - Threading support for socket functions added (should fix lagging)
*
*   v1.4 - 03/08/07
*       - reverse DNS support added (use "amx_showip_showhost 0" to disable it)
*
*   v1.3 - 03/06/07
*       - ADMIN_LEVEL macro added to restrict access if needed
*       - code cleanup and performance optimizations
*
*   v1.2 - 03/03/07
*       - Added colored chat messages support (use "amx_showip_usecolor 0" to disable it)
*
*   v1.1 - 03/01/07
*       - Added support for CIDR notation for ipdesc.ini file
*
*   v1.0 - 02/28/07
*	- Original show_ip plugin by W@lker /Yakutsk,Russia/ was rewritten for amxx
*/

#include <amxmodx>
#include <amxmisc>
#include "netdb"

// uncomment this to restrict access to amx_showip command and public announce
//#define ADMIN_LEVEL ADMIN_ADMIN

#define M_CONNECT "^x04%s ^x01(^x03%s^x01, ^x03%s^x01) is trying to connect"
#define MH_CONNECT "^x04%s ^x01(^x03%s^x01, ^x03%s^x01, ^x03%s^x01) is trying to connect"
#define M_DISCONNECT "^x04%s ^x01(^x03%s^x01, ^x03%s^x01) dropped"
#define MH_DISCONNECT "^x04%s ^x01(^x03%s^x01, ^x03%s^x01, ^x03%s^x01) dropped"


#define MAX_PLAYERS 32
#define MAX_DESCRIPTION_LEN 64
#define MAX_HOSTNAME_LEN 180

#define SHIFT_8  8
#define SHIFT_16 16
#define SHIFT_24 24

#define MAX_SUBNETS 1024 // Maximum records in ipdesc.ini file

enum ChatColor
{
	CHATCOLOR_YELLOW = 1, 	// Yellow
	CHATCOLOR_GREEN, 	// Green Color
	CHATCOLOR_TEAM_COLOR, 	// Red, grey, blue
	CHATCOLOR_GREY, 	// grey
	CHATCOLOR_RED, 		// Red
	CHATCOLOR_BLUE, 	// Blue
}

new g_TeamName[][] = 
{
	"",
	"TERRORIST",
	"CT",
	"SPECTATOR"
}


new g_SubnetIP[MAX_SUBNETS]
new g_SubnetMask[MAX_SUBNETS]
new g_SubnetDescription[MAX_SUBNETS][MAX_DESCRIPTION_LEN + 1]
new g_SubnetCount = 0

new g_IPDescription[MAX_PLAYERS + 1][MAX_DESCRIPTION_LEN + 1]
new g_HostName[MAX_PLAYERS + 1][MAX_HOSTNAME_LEN + 1]
new g_msgSayText
new g_msgTeamInfo

public plugin_init()
{
	register_plugin("Show IP", "1.55", "ahtoh")

	register_cvar("amx_showip_unknown", "UNKNOWN SUBNET") 	// default IP Subnet Description
	register_cvar("amx_showip_showhost", "1") 		// show Hostnames when player connects/disconnects

#if defined ADMIN_LEVEL
	register_concmd("amx_showip", "cmdShowIP", ADMIN_LEVEL,"- shows players IPs")
#else
	register_concmd("amx_showip", "cmdShowIP", ADMIN_ALL,"- shows players IPs")
#endif

	new configsDir[64]
	get_configsdir(configsDir, 63)
	format(configsDir, 63, "%s/ipdesc.ini", configsDir)

	loadSubnets(configsDir) // Load IP Descriptions from ipdesc.ini
	
	g_msgSayText = get_user_msgid("SayText")
	g_msgTeamInfo = get_user_msgid("TeamInfo") 
}

public cmdShowIP(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_HANDLED

	new players[32], inum, userip[16], name[32]
	
	get_players(players, inum)
	console_print(id, "^nPlayers on server:^n #  %-25.24s %-20s %-33.32s %s", "Nick", "IP", "Hostname", "Description")
	
	for (new a = 0; a < inum; ++a)
	{
		get_user_ip(players[a],userip,15,1)
		get_user_name(players[a], name, 31)
		console_print(id, "%2d  %-25.24s %-20s %-33.32s %s", players[a], name, userip, g_HostName[players[a]], g_IPDescription[players[a]])
	}
	
	console_print(id, "Total: %d", inum)
	
	return PLUGIN_HANDLED
}

public client_authorized(id)
{
	new userip[16]

	get_user_ip(id,userip,15,1) 

	thr_gethostbyaddr("client_showconnected", userip, g_HostName[id], MAX_HOSTNAME_LEN, id)
}
	
public client_showconnected(id)
{
	new userip[16]
	new name[32]

	get_user_ip(id,userip,15,1) 
	get_user_name(id,name,31)
	get_ip_description(userip, g_IPDescription[id])


#if defined ADMIN_LEVEL
	new players[32], inum
	
	for (new a = 0; a < inum; a++)
	{
		if (!access(players[a], ADMIN_LEVEL))
			continue

		if (get_cvar_num("amx_showip_showhost") == 0)
		{
			colorChat(players[a], CHATCOLOR_RED, M_CONNECT, name, userip, g_IPDescription[id])
		}
		else
		{
			colorChat(players[a], CHATCOLOR_RED, MH_CONNECT, name, userip, g_HostName[id], g_IPDescription[id])
		}
	}
#else
	if (get_cvar_num("amx_showip_showhost") == 0)
	{
		colorChat(0, CHATCOLOR_RED, M_CONNECT, name, userip, g_IPDescription[id])
	}
	else
	{
		colorChat(0, CHATCOLOR_RED, MH_CONNECT, name, userip, g_HostName[id], g_IPDescription[id])
	}
#endif
	
	return PLUGIN_CONTINUE
}  

public client_disconnect(id)
{
	new userip[16]
	new name[32]

	get_user_ip(id,userip,15,1) 
	get_user_name(id,name,31)

#if defined ADMIN_LEVEL
	new players[32], inum
	
	for (new a = 0; a < inum; a++)
	{
		if (!access(players[a], ADMIN_LEVEL))
			continue

		if (get_cvar_num("amx_showip_showhost") == 0)
		{
			colorChat(players[a], CHATCOLOR_RED, M_DISCONNECT, name, userip, g_IPDescription[id])
		}
		else
		{
			colorChat(players[a], CHATCOLOR_RED, MH_DISCONNECT, name, userip, g_HostName[id], g_IPDescription[id])
		}
	}
#else
	if (get_cvar_num("amx_showip_showhost") == 0)
	{
		colorChat(0, CHATCOLOR_RED, M_DISCONNECT, name, userip, g_IPDescription[id])
	}
	else
	{
		colorChat(0, CHATCOLOR_RED, MH_DISCONNECT, name, userip, g_HostName[id], g_IPDescription[id])
	}
#endif
	g_IPDescription[id][0] = 0	
	g_HostName[id][0] = 0	
		
	return PLUGIN_CONTINUE
} 

// ColorChat function, stolen from http://forums.alliedmods.net/showthread.php?t=45753
colorChat(id, ChatColor:color, const msg[], {Float,Sql,Result,_}:...)
{
	new team, index, MSG_Type
	new bool:teamChanged = false
	new message[192]

	switch(color)
	{
		case CHATCOLOR_YELLOW: // Yellow
		{
			message[0] = 0x01;
		}
		case CHATCOLOR_GREEN: // Green
		{
			message[0] = 0x04;
		}
		default: // Grey, Red, Blue
		{
			message[0] = 0x03;
		}
	}

	vformat(message[1], 190, msg, 4);

	if (id == 0)
	{
		index = findAnyPlayer();
		MSG_Type = MSG_ALL;
	}
	else
	{
		index = id;
		MSG_Type = MSG_ONE;
	}
	
	if (index != 0)
	{
		team = get_user_team(index);	

		if (color == CHATCOLOR_RED && team != 1)
		{
			messageTeamInfo(index, MSG_Type, g_TeamName[1])
			teamChanged = true
		}
		else if (color == CHATCOLOR_BLUE && team != 2)
		{
			messageTeamInfo(index, MSG_Type, g_TeamName[2])
			teamChanged = true
		}
		else if (color == CHATCOLOR_GREY && team != 0)
		{
			messageTeamInfo(index, MSG_Type, g_TeamName[0])
			teamChanged = true
		}
	
		messageSayText(index, MSG_Type, message);
			
		if (teamChanged)
		{
			messageTeamInfo(index, MSG_Type, g_TeamName[team]);
		}
	}
}

messageSayText(id, type, message[])
{
	message_begin(type, g_msgSayText, _, id)
	write_byte(id)		
	write_string(message)
	message_end()
}

messageTeamInfo(id, type, team[])
{
	message_begin(type, g_msgTeamInfo, _, id)
	write_byte(id)
	write_string(team)
	message_end()
}

findAnyPlayer()
{
	new players[32], inum
	
	get_players(players, inum)
	
	for (new a = 0; a < inum; ++a)
	{
		if(is_user_connected(players[a]))
			return players[a];
	}

	return 0
}

// Find subnet description
get_ip_description(const ip[16], description[MAX_DESCRIPTION_LEN + 1])
{
	new numIP = ip_to_num(ip)
	
	for (new i = 0; i < g_SubnetCount; i++)
	{
		if (apply_mask(numIP, g_SubnetMask[i]) == g_SubnetIP[i])
		{
			copy(description, MAX_DESCRIPTION_LEN, g_SubnetDescription[i])
			return 0
		}
	}

	get_cvar_string("amx_showip_unknown", description, MAX_DESCRIPTION_LEN)

	return -1
}

// IP to NUM
ip_to_num(const ip[16])
{
	new n1[4], n2[4], n3[4], n4[4]
	new tmpStr[16]
	
	copy(tmpStr, 15, ip)
	replace_all(tmpStr, 15, ".", " ")
	if (parse(tmpStr, n1, 3, n2, 3, n3, 3, n4, 3) != 4)
		return -1 // wrong ip address
		
	return (str_to_num(n1) << SHIFT_24) + (str_to_num(n2) << SHIFT_16) + (str_to_num(n3) << SHIFT_8) + str_to_num(n4)
}

// this rounds ip to its subnet by given mask
apply_mask(const numIP, const maskBit)
{
	if (maskBit < 0 || maskBit > 32)
		return -1 // wrong mask

	new shift = 32 - maskBit
	
	return (numIP >> shift) << shift 
}

loadSubnets(szFilename[])
{
	if (!file_exists(szFilename))
		return 0

	new line = 0, textline[256], len
	new subnet[24], subnetIP[16], subnetMask[8]
		
	while (g_SubnetCount < MAX_SUBNETS && (line = read_file(szFilename, line, textline, 255, len)))
	{
		if (len == 0 || equal(textline, ";", 1))
			continue // comment line

		strbreak(textline, subnet, 23, g_SubnetDescription[g_SubnetCount], MAX_DESCRIPTION_LEN)

		replace(subnet, 23, "/", " ")
		if (parse(subnet, subnetIP, 15, subnetMask, 7) != 2)
			continue // wrong subnet
		
		g_SubnetIP[g_SubnetCount] = ip_to_num(subnetIP)
		g_SubnetMask[g_SubnetCount] = str_to_num(subnetMask)
		
		++g_SubnetCount
	}
	
	return 1
}
